@@ -4,11 +4,13 @@
 
 use crate::grammar::Associativity;
 use crate::grammar::Grammar;
+use crate::grammar::Production;
 use crate::grammar::Symbol;
 use crate::lexer::Lexeme;
 use crate::parser::ParserColumn;
 use crate::parser::ParserState;
 use std::collections::HashMap;
+use std::collections::LinkedList;
 use std::rc::Rc;
 
 /// Representation of an AST
@@ -18,10 +20,12 @@ pub enum Tree {
     Leaf(Lexeme),
     /// Group of many [Tree::Leaf].
     Node {
-        /// Name of the grammar rule that produced this node.
-        kind:   String,
+        /// Name of the [GrammarRule](crate::grammar::GrammarRule) that produced this node.
+        rule_name:  Rc<String>,
+        /// Reference to the [Production] that produced this node.
+        production: Rc<Production>,
         /// Children of this Node.
-        leaves: Vec<Rc<Tree>>,
+        leaves:     Vec<Rc<Tree>>,
     },
 }
 
@@ -36,8 +40,12 @@ impl std::fmt::Display for Tree {
                 Tree::Leaf(lexeme) => {
                     writeln!(f, "{}{lexeme}", "  ".repeat(depth + 1))
                 }
-                Tree::Node { kind, leaves } => {
-                    let result = writeln!(f, "{}{kind}", "  ".repeat(depth));
+                Tree::Node { rule_name, leaves, production } => {
+                    let result = writeln!(
+                        f,
+                        "{}{rule_name} := {production}",
+                        "  ".repeat(depth)
+                    );
 
                     for leaf in leaves {
                         recurse(
@@ -60,6 +68,27 @@ impl std::fmt::Display for Tree {
     }
 }
 
+impl Tree {
+    /// Traverse the tree in pre-order.
+    pub fn traverse_in_pre_order(&self) -> LinkedList<&Tree> {
+        let mut todo: LinkedList<&Tree> = LinkedList::new();
+        let mut ordered: LinkedList<&Tree> = LinkedList::new();
+
+        todo.push_back(self);
+        ordered.push_front(self);
+        while !todo.is_empty() {
+            if let Some(Tree::Node { leaves, .. }) = todo.pop_front() {
+                for tree in leaves {
+                    ordered.push_front(tree);
+                    todo.push_back(tree);
+                }
+            }
+        }
+
+        ordered
+    }
+}
+
 pub(crate) fn build(
     grammar: &Grammar,
     lexemes: &[Lexeme],
@@ -124,7 +153,8 @@ fn build_forest_helper(
         }
 
         return vec![Rc::new(Tree::Node {
-            kind: (*state.name).clone(),
+            rule_name: state.rule_name.clone(),
+            production: state.production.clone(),
             leaves,
         })];
     }
@@ -155,7 +185,7 @@ fn build_forest_helper(
                 .iter()
                 .take_while(|state_partial| *state_partial != state)
                 .filter(|state_partial| {
-                    *state_partial.name == *name
+                    *state_partial.rule_name == *name
                         && (symbol_index > 0
                             || state_partial.start_column == state.start_column)
                         && satisfies_disambiguation(
@@ -200,12 +230,14 @@ fn satisfies_disambiguation(
     state_partial: &ParserState,
     state: &ParserState,
 ) -> bool {
-    if state_partial.production.symbols.len() == 3
-        && state.production.symbols.len() == 3
-    {
-        if let (Symbol::Rule(name_partial), Symbol::Rule(name)) =
-            (&state_partial.production.symbols[1], &state.production.symbols[1])
-        {
+    if let (Some(partial_index), Some(index)) = (
+        get_disambiguation(grammar, state_partial),
+        get_disambiguation(grammar, state),
+    ) {
+        if let (Symbol::Rule(name_partial), Symbol::Rule(name)) = (
+            &state_partial.production.symbols[partial_index],
+            &state.production.symbols[index],
+        ) {
             if let (Some(rule_partial), Some(rule)) =
                 (grammar.rules.get(name_partial), grammar.rules.get(name))
             {
@@ -242,3 +274,22 @@ fn satisfies_disambiguation(
 
     true
 }
+
+fn get_disambiguation(grammar: &Grammar, state: &ParserState) -> Option<usize> {
+    let mut disambiguations =
+        state.production.symbols.iter().enumerate().filter_map(
+            |(index, symbol)| {
+                if let Symbol::Rule(name) = symbol {
+                    if let Some(rule) = grammar.rules.get(name) {
+                        if rule.disambiguation.is_some() {
+                            return Some(index);
+                        }
+                    }
+                }
+
+                None
+            },
+        );
+
+    disambiguations.next()
+}
